#!/usr/local/bin/perl5 -w
#
# Purpose:  
#  To provide a very basic wrapper for small data files.
#
#  This data must be small enough to fit in memory, and is expected
#  to be accessible via a file handle with one line per object.
#  Data may be separated by either whitespace or commas.
#  Beginning/trailing whitespace will be ignored.
#
#     get_count($)       # return how many items there are
#     get_dims($)        # return embedding dimensionality
#     get_obj($$)        # return the object by 0-based index, as an
#                        # array of scalars
#
##########################################################################
#
# Package Header
#
##########################################################################

package SimpleWrapper;
require Exporter;

use strict;
use POSIX;

##########################################################################

BEGIN {
	use Exporter ();
	use Symbol;
	use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

	$VERSION     = 1.0;
	@ISA         = qw(Exporter);
	@EXPORT =  qw(
								new
                load_file
                get_count
                get_dims
                get_obj
						  );
	%EXPORT_TAGS = ();
	@EXPORT_OK   = ();

	return 1;
}



# new:  creates blessed SimpleWrapper objects with default params
#
sub new($@) {
	my $class    = shift;
	my $file_ref = shift;
	my $self     = +{};

	bless $self, $class;

	$self->{"data"}   = +{};
	$self->{"params"} = +{};

	if (defined($file_ref)) {
		$self->load_file($file_ref);
	}
	
	return $self;	
}



# Load the file.  There's some minor munging to handle both
# whitespace and commas.
sub load_file($$) {
	my $self     = shift;
	my $file_ref = shift;
	my $dims     = undef;
	my $count    = 0;
	my $line     = undef;
	my @obj      = undef;

	$self->{"data"} = +{};
	
	while (defined($line = <$file_ref>)) {
		$line =~ s/,/ /g;
		$line =~ s/\s\s+/ /g;
		$line =~ s/^\s+//;
		chomp($line);

		(defined($line) && ($line ne "")) || next;

		@obj = split /\s/, $line;
		$self->{"data"}->{$count} = +[ @obj ];
		$count++;

		if (!defined($dims)) {
			$dims = scalar @obj;
		} else {
			($dims == (scalar @obj)) || die "Irregularly sized objects...";
		}
	}

	$self->{"params"}->{"count"} = $count;
	$self->{"params"}->{"dims"}  = $dims;

	return $self;
}




# Return the stored count... assumes the data has already been
# loaded.
sub get_count($) {
	my $self = shift;

	defined($self->{"params"}->{"count"}) || die "Can't get_count with 0 data";

	return $self->{"params"}->{"count"};
}



# Return the embedding dimensionality... assumes the data has already been
# loaded.
sub get_dims($) {
	my $self = shift;

	defined($self->{"params"}->{"dims"}) || die "Can't get_dims with 0 data";

	return $self->{"params"}->{"dims"};
}



# Return a specific object, by index.
sub get_obj($$) {
	my $self  = shift;
	my $index = shift;

	defined($self->{"data"}->{$index}) || die "Unexpected index '$index'";

	return @{$self->{"data"}->{$index}};
}
              
